/*
 * comm.c
 *
 * Created: 2016-05-11 13:12:59
 *  Author: dsosnowski
 */ 


#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "serial.h"
#include "defs.h"
#include "gsm.h"
#include "comm.h"
#include "exe.h"
#include "settings.h"

#define COMM_BUFF_LENGHT				128
#define COMM_COMMANDS_COUNT				9


const char text_comm_out1[] = "Out";
const char text_comm_out2[] = "Wyj";
const char text_comm_in1[]  = "Input";
const char text_comm_in2[]  = "Wej";
const char text_comm_pass1[]  = "Pass";
const char text_comm_pass2[]  = "Haslo";
const char text_comm_sett[]  = "Alarm";
const char text_comm_time1[]  = "Time";
const char text_comm_time2[]  = "Czas";



char *commands[COMM_COMMANDS_COUNT];
char commands_code[COMM_COMMANDS_COUNT];

#define COMMANDS_END_CHAR				'.'
//#define COMMANDS_OUT_CODE				'O'
//#define COMMANDS_ALARM_CODE			'A'
//#define COMMANDS_IN_CODE				'I'
//#define COMMANDS_PASS_CODE			'P'
//#define COMMANDS_SETT_CODE			'S'
//#define COMMANDS_TIME_CODE			'T'


#define COMMANDS_BREAK_NEXT				1
#define COMMANDS_BREAK_END				2		

struct {
	char buffer[COMM_BUFF_LENGHT];
	char new;
	char command;
	int16_t param;
	char action;
	char channel;
	char pass;
} comm;


#define COMM_ACTION_IDLE				0
#define COMM_ACTION_ENABLE				1
#define COMM_ACTION_DISABLE				2
#define COMM_ACTION_CHECK				3

struct {
	char buffer[COMM_BUFF_LENGHT+1];
	char status;
	char index;
} resp;


//-----------------------------------------------------
void CommInit(void) {
	commands[0] = text_comm_out1;
	commands_code[0] = COMMANDS_OUT_CODE;
	commands[1] = text_comm_out2;
	commands_code[1] = COMMANDS_OUT_CODE;
	commands[2] = text_comm_in1;
	commands_code[2] = COMMANDS_IN_CODE;
	commands[3] = text_comm_in2;
	commands_code[3] = COMMANDS_IN_CODE;
	commands[4] = text_comm_pass1;
	commands_code[4] = COMMANDS_PASS_CODE;
	commands[5] = text_comm_pass2;
	commands_code[5] = COMMANDS_PASS_CODE;
	commands[6] = text_comm_sett;
	commands_code[6] = COMMANDS_SETT_CODE;
	commands[7] = text_comm_time1;
	commands_code[7] = COMMANDS_TIME_CODE;
	commands[8] = text_comm_time2;
	commands_code[8] = COMMANDS_TIME_CODE;
}

//-----------------------------------------------------
void CommGetMsg(char offset) {
	char status = 0;
	char *pt = SerialRBufferPt(offset);
	char i = 0;
	
	if (comm.new) status = 2;		//skip
	
	while (status < 2) {
		if (status == 0) {
			if (*pt == 0) status = 1;
			if ((*pt == '\r') && (*(pt+1) == '\n')) status = 1;
		}
		
		if (status == 0) {
			comm.buffer[i] = *pt;
			pt++;
			i++;
			comm.buffer[i] = 0;
			comm.new = 1;
		} else {
			comm.buffer[i] = 0;
			i++;			
		}
		
		if (i >= COMM_BUFF_LENGHT) status = 2;		
	}
}


//-----------------------------------------------------
void CommProc(void) {
	char *text;
	char buffer_index;
	char command_index;
	char result;
		
	if (comm.new) {
		comm.new = 0;
		comm.pass = SettPassCheck(0);
		buffer_index = 0;
				
		result = 0;		
		while (result == 0) {
						
			command_index = 0;
			comm.command = 0;
			result = 0;
			while (result == 0) {
				
				if (comm.buffer[buffer_index] == ' ') buffer_index++;
				result = CommTextBreak(buffer_index);
				

				if (result == 0){
					text = commands[command_index];
					if (CommTextComp(buffer_index, text) == 1) {
						
						comm.param = 0;
						comm.action = 0;
						comm.command = commands_code[command_index];
						buffer_index += CommTextSkip(buffer_index);
						
						result = CommTextBreak(buffer_index);
						
						if (result == 0){
							if (CommTextValueCheck(buffer_index)) {
								comm.param = CommTextValue(buffer_index);
								buffer_index += CommTextSkip(buffer_index);
							}
						}

						result = CommTextBreak(buffer_index);
						
						if (result == 0){
							if (CommTextComp(buffer_index, "on") == 1) comm.action = COMM_ACTION_ENABLE;
							if (CommTextComp(buffer_index, "wl") == 1) comm.action = COMM_ACTION_ENABLE;
							if (CommTextComp(buffer_index, "set") == 1) comm.action = COMM_ACTION_ENABLE;
							if (CommTextComp(buffer_index, "off") == 1) comm.action = COMM_ACTION_DISABLE;
							if (CommTextComp(buffer_index, "wyl") == 1) comm.action = COMM_ACTION_DISABLE;
							if (CommTextComp(buffer_index, "reset") == 1) comm.action = COMM_ACTION_DISABLE;
							if (CommTextComp(buffer_index, "?") == 1) comm.action = COMM_ACTION_CHECK;
							if (CommTextComp(buffer_index, "s") == 1) comm.action = 4;
							if (CommTextComp(buffer_index, "m") == 1) comm.action = 5;
							if (CommTextComp(buffer_index, "h") == 1) comm.action = 6;
							if (CommTextComp(buffer_index, "g") == 1) comm.action = 6;
							
							buffer_index += CommTextSkip(buffer_index);
						}

						result = CommTextBreak(buffer_index);
					}
				}
				
				if (comm.command) {
					if ((comm.command == COMMANDS_PASS_CODE) || (comm.pass > 0)) {
						CommAction();
					}
					comm.command = 0;
				}
				
				command_index++;
				if (command_index >= COMM_COMMANDS_COUNT) {			//unknown command
					buffer_index = CommTextFind(buffer_index, COMMANDS_END_CHAR, 1);
					if (buffer_index > 0) {
						result = COMMANDS_BREAK_NEXT;
					} else {
						result = COMMANDS_BREAK_END;				
					}
				}				
			}
			
			if (result == COMMANDS_BREAK_NEXT) {
				result = 0;
				buffer_index++;
			} else {
				result = COMMANDS_BREAK_END;
			}	
		}
		comm.pass = 0;
	}
	
}

//-----------------------------------------------------
char CommTextFind(char offset, char data, char num) {
	char index = offset;
	char status = 0;
	while (status == 0) {
		if (index < COMM_BUFF_LENGHT) {
			if (comm.buffer[index] == data) num--;
			if (num == 0)  {
				status = 1;
			} else {
				index++;
			}
		} else status = 2;
	}
	if (status == 1) return index;
	return 0;		
}

//-----------------------------------------------------
char CommTextComp(char i, char *model) {
	char pass = 1;
	
	while (*model > 0) {
		if (comm.buffer[i] != *model) {
			pass = 0;
			break;
		}
		i++;
		model++;
	}
	
	if (pass) {
		if ((comm.buffer[i] == ' ') || (comm.buffer[i] == COMMANDS_END_CHAR) || (comm.buffer[i] == 0)) {
			return 1;
		}
	}
	
	return 0;
}

//-----------------------------------------------------
char CommTextSkip(char i) {
	char result = 0;
	char status = 0;
	while (status == 0) {
		if ((comm.buffer[i] > 0) && (comm.buffer[i] != COMMANDS_END_CHAR)){
			if ((comm.buffer[i] == ' ') && (comm.buffer[i+1] != ' ')) status = 1;
			result++;
			i++;
		} else {
			status = 1;	
		}
	}
	return result;
}

//-----------------------------------------------------
char CommTextBreak(char i) {
	if (comm.buffer[i] == COMMANDS_END_CHAR) return COMMANDS_BREAK_NEXT;
	if (comm.buffer[i] == 0) return COMMANDS_BREAK_END;
	return 0;
}


//------------------------------------------
char CommTextValueCheck(char i){
	if ((comm.buffer[i] >= '0') && (comm.buffer[i] <= '9')) return 1;
	return 0;
}

//------------------------------------------
uint16_t CommTextValue(char i){
	uint16_t val = 0;
	char status = 1;
	char n = 0;

	if ((comm.buffer[i] >= '0') && (comm.buffer[i] <= '9')) {
		status = 0;
		val = 0;

		while (status == 0) {
			if ((comm.buffer[i] >= '0') && (comm.buffer[i] <= '9')) {
				val *= 10;
				val += (comm.buffer[i] - '0');
				i++;
				n++;
			} else {
				status = 1;
			}
			if (n >= 4) status = 1;
		}	

		if (comm.buffer[i] == 'm') {
			if (val > 999) val = 999;
			val *= 60;
		}
		if ((comm.buffer[i] == 'h') || (comm.buffer[i] == 'g')) {
			if (val > 12) val = 12;
			val *= 3600;
		}
	}
	return val;
}

//-----------------------------------------------------
void CommActionAdd(char command, int32_t param, char action) {
	comm.command = command;
	comm.param = param;
	comm.action = action;
	CommAction();
}

//-----------------------------------------------------
void CommAction(void) {
	if (comm.command == COMMANDS_PASS_CODE) {
		if (comm.pass == 0) {
			comm.pass = SettPassCheck(comm.param);		
		}
		    		
		if (comm.action == COMM_ACTION_CHECK) {
			resp.status |= RESP_STATUS_PASS;			
		}

		if (comm.pass > 0){
			if (comm.action == COMM_ACTION_ENABLE) {
				SettPassNew(comm.param);
				resp.status |= RESP_STATUS_PASS;
			}			
			if (comm.action == COMM_ACTION_DISABLE) {
				SettPassNew(0);
				resp.status |= RESP_STATUS_PASS;
			}
		}
	}
	
	if (comm.command == COMMANDS_ALARM_CODE) {
		if (comm.param & RESP_STATUS_ALARM1) resp.status |= RESP_STATUS_ALARM1;
		if (comm.param & RESP_STATUS_ALARM2) resp.status |= RESP_STATUS_ALARM2;
	}

	if (comm.command == COMMANDS_OUT_CODE) {	
		if ((comm.param > 0) && (comm.param <= 2)) {
			comm.channel = (comm.param - 1);
		} else {
			comm.channel = 0;
		}
			if (comm.action == COMM_ACTION_ENABLE) {
				ExeOutputSet(comm.channel);
				resp.status |= RESP_STATUS_OUTPUTS;
			}
			if (comm.action == COMM_ACTION_DISABLE) {
				ExeOutputOff(comm.channel);
				resp.status |= RESP_STATUS_OUTPUTS;
			}
			if (comm.action == COMM_ACTION_CHECK) {
				resp.status |= RESP_STATUS_OUTPUTS;
			}
	}
	
	if (comm.command == COMMANDS_TIME_CODE) {
		if ((comm.channel >= 0) && (comm.channel < 2)) {

			if (comm.action == COMM_ACTION_IDLE) {
				ExeDelaySet(comm.channel, comm.param);
			}
			if (comm.action == COMM_ACTION_ENABLE) {
				ExeDelaySet(comm.channel, comm.param);
				ExeDelayStateSet(comm.channel, 1);
			}
			if (comm.action == COMM_ACTION_DISABLE) {
				ExeDelaySet(comm.channel, comm.param);
				ExeDelayStateSet(comm.channel, 0);
			}
			resp.status |= RESP_STATUS_OUTPUTS;
		}
	}
		
	if (comm.command == COMMANDS_IN_CODE) {
		if (comm.action == COMM_ACTION_CHECK) {
			resp.status |= RESP_STATUS_INPUTS;	
		}
	}

	if (comm.command == COMMANDS_SETT_CODE) {
		if ((comm.param > 0) && (comm.param <= 2)) {
			comm.channel = (comm.param - 1);
		} else {
			comm.channel = 0;
		}		
		
			if (comm.action == COMM_ACTION_ENABLE) {
				SettNumNew(comm.channel, GsmSmsNumGet());
				resp.status |= (RESP_STATUS_SETT1);
			}
			if (comm.action == COMM_ACTION_DISABLE) {
				SettNumReset(comm.channel);
				resp.status |= (RESP_STATUS_SETT2);
			}
			if (comm.action == COMM_ACTION_CHECK) {
				resp.status |= (RESP_STATUS_SETT1 | RESP_STATUS_SETT2);
			}			

	}
}
	
//-----------------------------------------------------
void CommRespProc(void) {
	static char *number;
	
	if (resp.status > 0) {
		CommRespNew(1);
		number = 0;
		
		if (resp.status & RESP_STATUS_PASS) {
			resp.status &= ~RESP_STATUS_PASS;
			CommRespAdd("Password ", 0);
			if (SettPassCheck(0) == 1) {
				CommRespAddState(0);
			} else {
				CommRespAddState(1);
				//CommRespAddValue(SettPassGet());				
			}
			CommRespAdd(". ", 0);
		}
	
		if (resp.status & RESP_STATUS_ALARM1) {
			resp.status &= ~RESP_STATUS_ALARM1;
			number = SettNumGet(0);
			CommRespAdd("ALARM ", 0);
			CommRespAddChannel(0);
			CommRespAdd(" !!!. ", 0);
		}
		if (resp.status & RESP_STATUS_ALARM2) {
			resp.status &= ~RESP_STATUS_ALARM2;
			number = SettNumGet(1);
			CommRespAdd("ALARM ", 0);
			CommRespAddChannel(1);
			CommRespAdd(" !!!. ", 0);
		}


		if (resp.status & RESP_STATUS_OUTPUTS) {
			resp.status &= ~RESP_STATUS_OUTPUTS;
			CommRespAdd("Out ", 0);
			CommRespAddChannel(0);
			CommRespAddState(ExeOutputGet(0));
			CommRespAdd(".", 0);
			
			if (ExeDelayCheck(0)) {
				CommRespAdd(" Time ", 0);
				CommRespAddTime(ExeDelayGet(0));
				CommRespAdd(" ", 0);
				CommRespAddState(ExeDelayStateGet(0));
				CommRespAdd(".", 0);
			}
			
			CommRespAdd(" Out ", 0);
			CommRespAddChannel(1);
			CommRespAddState(ExeOutputGet(1));
			CommRespAdd(".", 0);

			if (ExeDelayCheck(1)) {
				CommRespAdd(" Time ", 0);
				CommRespAddTime(ExeDelayGet(1));
				CommRespAdd(" ", 0);
				CommRespAddState(ExeDelayStateGet(1));
				CommRespAdd(".", 0);
			}
		}
		
		if (resp.status & RESP_STATUS_INPUTS) {
			resp.status &= ~RESP_STATUS_INPUTS;
			CommRespAdd(" Input ", 0);
			CommRespAddChannel(0);
			CommRespAddState(ExeInputGet(0));
			//CommRespAdd(". ", 0);
			CommRespAdd(". Input ", 0);
			CommRespAddChannel(1);
			CommRespAddState(ExeInputGet(1));
			CommRespAdd(". ", 0);
		}
		
		if (resp.status & RESP_STATUS_SETT1) {
			resp.status &= ~RESP_STATUS_SETT1;
			CommRespAdd("Alarm ", 0);
			CommRespAddChannel(0);
			
			if (SettNumAlarmCheck(0) > 0) {
				CommRespAdd("num:", 0);
				CommRespAdd(SettNumGet(0), SETT_NUM_LENGTH);				
			} else {
				CommRespAdd("off", 0);				
			}
			CommRespAdd(". ", 0);	
		}
		
		if (resp.status & RESP_STATUS_SETT2) {
			resp.status &= ~RESP_STATUS_SETT2;
			CommRespAdd("Alarm ", 0);
			CommRespAddChannel(1);
			
			if (SettNumAlarmCheck(1) > 0) {
				CommRespAdd("num:", 0);
				CommRespAdd(SettNumGet(1), SETT_NUM_LENGTH);
			} else {
				CommRespAdd("off", 0);
			}
			CommRespAdd(". ", 0);
		}
	}

	if (CommRespNew(0) > 0) {
		if (GsmSmsPrepare(resp.buffer, number) == 1) {
			CommRespNew(1);
		}
	}
}

//-----------------------------------------------------
char CommRespNew(char action) {
	if (action > 0) {
		resp.index = 0;
		resp.buffer[resp.index] = 0;	
	}
	return resp.index; 
}

//-----------------------------------------------------
void CommRespData(char data) {
	if (resp.index < COMM_BUFF_LENGHT)
	{
		resp.buffer[resp.index] = data;
		resp.index++;
		resp.buffer[resp.index] = 0;		
	}
}

//-----------------------------------------------------
char CommRespAdd(char *src, char len) {
	char status = 0;
	
	if (*src == 0) status = 1;
	
	while (status == 0) {
		if (resp.index < COMM_BUFF_LENGHT) {
			resp.buffer[resp.index] = *src;
			resp.index++;
			src++;
			resp.buffer[resp.index] = 0;			

			if (len > 0) {
				len--;
				if (len == 0) status = 1;
			}
		} else {
			status = 2;			//not enough sms space
		}
		
		if (len == 0) {
			if (*src == 0) status = 1;
		}
	}
	return status;
}

//-----------------------------------------------------
void CommRespAddChannel(char channel) {
	CommRespData(channel + '1');
	CommRespData(' ');
}

//-----------------------------------------------------
void CommRespAddState(char state) {
	if (state == 0) 
		CommRespAdd("off", 0);	
	else 
		CommRespAdd("on", 0);
	
	//if (state == 1) CommRespAdd("on ", 0);
}

//-----------------------------------------------------
void CommRespAddValue(uint16_t value) {
	uint16_t temp = 10000;
	char digit;
	char blank = 1;

	while (temp)
	{
		digit = value / temp;
		value = value % temp;
		
		if (temp == 1) blank = 0;
		if (digit) blank = 0;
		
		if(!(blank)) CommRespData(digit + '0');
		//else SerialSendChar(' ');

		temp = temp / 10;
	}
	//CommRespData(' ');
}

//-----------------------------------------------------
void CommRespAddTime(uint16_t value) {
	uint16_t part;
	char active = 0;
	
	if (value >= 3600) {
		part = value / 3600;
		CommRespAddValue(part);
		CommRespData('h');
		value %= 3600;
		active = 1;
	}
	if (value >= 60) active = 1;
	if (active == 1) {
		part = value / 60;
		CommRespAddValue(part);
		CommRespData('m');
		value %= 60;
	}
	CommRespAddValue(value);
	CommRespData('s');
}