/*
 * gsm800.c
 *
 * Created: 2016-05-10 10:30:08
 *  Author: dsosnowski
 */ 

#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "wdog.h"
#include "defs.h"
#include "serial.h"
#include "gsm.h"
#include "comm.h"
#include "exe.h"
#include "settings.h"

void HardInit(void);


//---------------------------------------------------


#define SYSTEM_RESPONSE_DELAY			(10000/MAIN_DELAY)



struct {
	uint16_t task;
	uint16_t led;
	uint16_t resp_delay;
	char resp_status;
} system;


//---------------------------------------------------
int main(void) {
	char n;
	char prev;	
		
		WdtSet(7);
		HardInit();
		ExeInit();
		SettInit();
		SerialInit();
		GsmInit();
		CommInit();
		_delay_ms(100);
		sei();		
		ExeCheckFactory();
		WdtReset();
		while(1)
	
    while(1) {	
			
		GsmProc();
		
		CommProc();
		
		ExeProc();

		if (ExeActionGet() > 0) {
			system.led = 0;
			n = ExeActionGet();
			if (n & EXE_SWITCH1_MASK) ExeOutputToggle(0);
			if (n & EXE_SWITCH2_MASK) ExeOutputToggle(1);
			if (n & EXE_ALARM1_MASK) {
				if (SettNumAlarmCheck(0) > 0) {
					system.resp_status |= RESP_STATUS_ALARM1;
				}
			}
			if (n & EXE_ALARM2_MASK) {
				if (SettNumAlarmCheck(1) > 0) {
					system.resp_status |= RESP_STATUS_ALARM2;
				}
			}
			ExeActionClear();
		}

		if (system.resp_delay > 0)  {
			system.resp_delay--;
			
			if (system.led >= (2 * MAIN_LED_PULSE)) {
				system.led = 0;
			}
			
			system.resp_status &= ~(prev);
			
		} else {
			prev = 0;
			
			if (system.resp_status) {
				CommActionAdd(COMMANDS_ALARM_CODE, system.resp_status, 0);
				prev = system.resp_status;
				system.resp_status = 0;
				system.resp_delay = SYSTEM_RESPONSE_DELAY;
			}
		}
							
		CommRespProc();
		
		
		if (system.led == 0) {
			LED_PORT |= (1<<LEDG_PIN);
		}
		if (system.led == MAIN_LED_PULSE) {
			LED_PORT &= ~(1<<LEDG_PIN);
		}
		system.led++;
		if (system.led >= MAIN_LED_PERIOD) {
			system.led = 0;
		}
		
        _delay_ms(MAIN_DELAY);
		
		WdtReset();		
    }
}

//---------------------------------------------------
void HardInit(void) {
	ACSR = (1<<ACD);	
	LED_DDR |= (1<<LEDG_PIN | 1<<LEDO_PIN | 1<<LEDR_PIN);
}