/*
 * settings.c
 *
 * Created: 2016-05-17 07:56:26
 *  Author: dsosnowski
 */ 


#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "serial.h"
#include "defs.h"
#include "gsm.h"
#include "comm.h"
#include "exe.h"
#include "settings.h"
#include "ee_mem.h"



//#define SETT_NUM_LENGTH		9
//#define SETT_NUM_OF_NUM		2
#define SETT_NUM_MEM		(SETT_NUM_LENGTH * SETT_NUM_OF_NUM)

#define SETT_EE_STARTADR	8
#define SETT_EE_LENGTH		(2 + SETT_NUM_MEM)


//-------------------------------------------------------------
struct {
	uint16_t pass;
	char numbers[SETT_NUM_MEM];
} sett;


//-------------------------------------------------------------
void SettInit(void){
	char i;
	char *pt;
	
	EERestore(&sett.pass, SETT_EE_STARTADR, SETT_EE_LENGTH);
	
	if (sett.pass > 9999) sett.pass = 0;
	
	pt = &sett.numbers[0];
	for (i=0; i<SETT_NUM_MEM; i++) {
		if ((*pt < '0') || (*pt > '9')) *pt = 0;
	}	
}

//-------------------------------------------------------------
void SettSave(void){
	cli();
	EEStore(&sett.pass, SETT_EE_STARTADR, SETT_EE_LENGTH);
	sei();	
}

//-------------------------------------------------------------
char SettPassCheck(uint16_t val){
	if (sett.pass > 0) {
		if (val == sett.pass) return 1;	
	} else {
		return 1;
	}
	return 0;
}

//-------------------------------------------------------------
void SettPassNew(uint16_t val){
	if (val > 9999) val = 0;
	cli();
	sett.pass = val;
	EEStore(&sett.pass, SETT_EE_STARTADR, 2);
	sei();
}

//-------------------------------------------------------------
uint16_t SettPassGet(void){
	return sett.pass;
}

//-------------------------------------------------------------
char *SettNumGet(char val){
	if (val < SETT_NUM_OF_NUM) {
		return &sett.numbers[(val * SETT_NUM_LENGTH)];
	}
	return 0;
}

//-------------------------------------------------------------
char SettNumAlarmCheck(char val){
	char i;
	char correct = 0;
	char *pt;
	
	if (val < SETT_NUM_OF_NUM) {
		pt = &sett.numbers[(val * SETT_NUM_LENGTH)];
		correct = 0;
		
		for(i=0; i<SETT_NUM_LENGTH; i++) {
			if (*pt == '0') correct++;			
			if ((*pt < '0') || (*pt > '9')) correct = SETT_NUM_LENGTH;
		}
	}
	if (correct < SETT_NUM_LENGTH) return 1;
	return 0;
}	
	
//-------------------------------------------------------------
void SettNumNew(char val, char *src){
	char i;
	char correct;
	char *pt;
	
	if (val < SETT_NUM_OF_NUM) {
		pt = &sett.numbers[(val * SETT_NUM_LENGTH)];
		correct = 0;
		
		for(i=0; i<SETT_NUM_LENGTH; i++) {
			*pt = *src;
			if (*pt == '0') correct++;
			if ((*pt < '0') || (*pt > '9')) correct = SETT_NUM_LENGTH;
			pt++;
			src++;
		}
		
		if (correct < SETT_NUM_LENGTH) SettSave();
		SettInit();
	}
}

//-------------------------------------------------------------
void SettNumReset(char val){
	char i;
	char *pt;
	
	if (val < SETT_NUM_OF_NUM) {
		pt = &sett.numbers[val * SETT_NUM_LENGTH];
		
		for(i=0; i<SETT_NUM_LENGTH; i++) {
			*pt = 0;
			pt++;
		}
	}
}
